@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2013-2019 Kai Pastor
+ *    Copyright 2013-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -95,7 +95,7 @@ public:
 	explicit Object(Type type, const Symbol* symbol = nullptr);
 	
 	/** Creates an empty object with the given type, symbol, coords and (optional) map. */
-	explicit Object(Type type, const Symbol* symbol, const MapCoordVector& coords, Map* map = nullptr);
+	explicit Object(Type type, const Symbol* symbol, MapCoordVector coords, Map* map = nullptr);
 	
 protected:
 	/**
@@ -234,7 +234,7 @@ public:
 	 * symbol type the coord is
 	 * (important for combined symbols which can have areas and lines).
 	 */
-	int isPointOnObject(const MapCoordF& coord, float tolerance, bool treat_areas_as_paths, bool extended_selection) const;
+	int isPointOnObject(const MapCoordF& coord, qreal tolerance, bool treat_areas_as_paths, bool extended_selection) const;
 	
 	/**
 	 * Checks if a path point (excluding curve control points) is included in the given box.
@@ -485,7 +485,7 @@ public:
 	explicit PathObject(const Symbol* symbol = nullptr);
 	
 	/** Constructs a PathObject, assigning initial coords and optionally the map pointer. */
-	explicit PathObject(const Symbol* symbol, const MapCoordVector& coords, Map* map = nullptr);
+	explicit PathObject(const Symbol* symbol, MapCoordVector coords, Map* map = nullptr);
 	
 	/**
 	 * Constructs a PathObject, assigning initial coords from a single piece of a line.
@@ -679,37 +679,50 @@ public:
 	// Operations
 	
 	/**
-	 * Calculates the closest point on the path to the given coordinate,
-	 * returns the squared distance of these points and PathCoord information
-	 * for the point on the path.
+	 * Calculates the closest point on the path to the given coordinate.
 	 * 
 	 * This does not need to be an existing path coordinate. This method is
 	 * usually called to find the position on the path the user clicked on.
-	 * part_index can be set to a valid part index to constrain searching
+	 * The parameters start_index and end_index can be set to constrain searching
 	 * to this specific path part.
-	 * 
-	 * \todo Convert out_distance_sq to double (so avoiding conversions).
-	 * \todo Return PathCoord rather than writing to the provided reference.
 	 */
-	void calcClosestPointOnPath(
-	        MapCoordF coord,
-	        float& out_distance_sq,
-	        PathCoord& out_path_coord,
+	ClosestPathCoord findClosestPointTo(
+	        const MapCoordF& coord,
 	        MapCoordVector::size_type start_index = 0,
 	        MapCoordVector::size_type end_index = std::numeric_limits<PathPartVector::size_type>::max()
 	) const;
 	
 	/**
-	 * Calculates the closest control point coordinate to the given coordinate,
-	 * returns the squared distance of these points and the index of the control point.
+	 * Calculates a border path with the closest point to the given coordinate.
 	 * 
-	 * \todo Convert out_distance_sq to double (so avoiding conversions).
-	 * \todo Return index rather than writing to the provided reference.
+	 * This function operates on path objects with a line symbol which has
+	 * visible borders, or with a combined symbol which features such a line
+	 * symbol. For all other symbols, the return value has an empty border.
+	 * 
+	 * The offsets of the border paths are determined by the main line width.
+	 * If the border line is continuous (not dashed), border line width and
+	 * offset are added so that the returned path is at the center of the
+	 * border line.
+	 * 
+	 * In addition to the MapCoordF input parameter, this function takes a
+	 * PathCoord input parameter which serves as a key to determining candidate
+	 * border path locations. This PathCoord is meant to be looked up with
+	 * findClosestPointTo() before calling this function.
+	 * 
+	 * If no point closer than distance_bound_squared, the return value has an
+	 * empty border.
 	 */
-	void calcClosestCoordinate(
-	        MapCoordF coord,
-	        float& out_distance_sq,
-	        MapCoordVector::size_type& out_index) const;
+	ClosestBorderPathCoord findClosestPointOnBorder(
+	        const MapCoordF& coord,
+	        const PathCoord& path_coord,
+	        double distance_bound_squared
+	) const;
+	
+	/**
+	 * Finds the index of the closest control point coordinate to the given coordinate.
+	 */
+	MapCoordVector::size_type findClosestCoordinate(const MapCoordF& coord) const;
+	
 	
 	/**
 	 * Splits the path at the position given by path_coord.
@@ -748,7 +761,7 @@ public:
 	
 	/**
 	 * Connects the given parts, optionally merging the end coordinates at the
-	 * center position, and copying over the coordindates from other.
+	 * center position, and copying over the coordinates from other.
 	 */
 	void connectPathParts(
 	        PathPartVector::size_type part_index,
@@ -840,8 +853,8 @@ public:
 	
 	/** See Object::isPointOnObject() */
 	int isPointOnPath(
-	        MapCoordF coord,
-	        float tolerance,
+	        const MapCoordF& coord,
+	        qreal tolerance,
 	        bool treat_areas_as_paths,
 	        bool extended_selection
 	) const;
@@ -934,7 +947,7 @@ protected:
 	 * to calculate the current cost. Evaluates the distance between p0 ... p3
 	 * and the reference path.
 	 */
-	static float calcBezierPointDeletionRetainingShapeCost(
+	static double calcBezierPointDeletionRetainingShapeCost(
 		MapCoord p0,
 		MapCoordF p1,
 		MapCoordF p2,
@@ -1078,9 +1091,9 @@ struct ObjectPathCoord : public PathCoord
 	 * 
 	 * \return The squared distance of these points.
 	 * 
-	 * \see PathObject::calcClosestPointOnPath
+	 * \see PathObject::findClosestPointTo
 	 */
-	float findClosestPointTo(const MapCoordF& map_coord);
+	double findClosestPointTo(const MapCoordF& map_coord);
 };
 
 
@@ -1264,11 +1277,11 @@ ObjectPathCoord::ObjectPathCoord(PathObject* object, MapCoordVector::size_type i
 
 
 inline
-float ObjectPathCoord::findClosestPointTo(const MapCoordF& map_coord)
+double ObjectPathCoord::findClosestPointTo(const MapCoordF& map_coord)
 {
-	float distance_sq;
-	object->calcClosestPointOnPath(map_coord, distance_sq, *this);
-	return distance_sq;
+	auto const closest = object->findClosestPointTo(map_coord);
+	static_cast<PathCoord&>(*this) = closest.path_coord;
+	return closest.distance_squared;
 	
 }
 
