@@ -1,5 +1,5 @@
 /*
- *    Copyright 2016-2019 Kai Pastor
+ *    Copyright 2016-2020 Kai Pastor
  *
  *    Some parts taken from file_format_oc*d8{.h,_p.h,cpp} which are
  *    Copyright 2012 Pete Curtis
@@ -545,7 +545,7 @@ QTextCodec* OcdFileExport::determineEncoding<Ocd::Custom8BitEncoding>()
 	auto encoding = codecFromSettings();
 	if (!encoding)
 	{
-		addWarning(tr("Encoding '%1' is not available. Check the settings.")
+		addWarning(::OpenOrienteering::OcdFileExport::tr("Encoding '%1' is not available. Check the settings.")
 		           .arg(Settings::getInstance().getSetting(Settings::General_Local8BitEncoding).toString()));
 		encoding = QTextCodec::codecForLocale();
 	}
@@ -573,8 +573,9 @@ bool OcdFileExport::exportImplementation()
 		
 	default:
 		throw FileFormatException(
-		            Exporter::tr("Could not write file: %1").
-		            arg(tr("OCD files of version %1 are not supported!").arg(ocd_version))
+		            ::OpenOrienteering::Exporter::tr("Could not write file: %1").
+		            arg(::OpenOrienteering::OcdFileExport::tr("OCD files of version %1 are not supported!")
+		                .arg(ocd_version))
 		            );
 	}
 }
@@ -612,8 +613,6 @@ void setupFileHeaderGeneric(quint16 actual_version, Ocd::FileHeaderGeneric& head
 template<class Format>
 bool OcdFileExport::exportImplementation()
 {
-	addWarning(QLatin1String("OcdFileExport: WORK IN PROGRESS, FILE INCOMPLETE"));
-	
 	OcdFile<Format> file;
 	
 	custom_8bit_encoding = determineEncoding<typename Format::Encoding>();
@@ -659,7 +658,7 @@ MapCoord OcdFileExport::calculateAreaOffset()
 		    && objects_extent.height() < ocd_bounds.height())
 		{
 			// The extent fits into the limited area.
-			addWarning(tr("Coordinates are adjusted to fit into the OCAD 8 drawing area (-2 m ... 2 m)."));
+			addWarning(::OpenOrienteering::OcdFileExport::tr("Coordinates are adjusted to fit into the OCAD 8 drawing area (-2 m ... 2 m)."));
 			area_offset = objects_extent.center();
 		}
 		else
@@ -670,7 +669,7 @@ MapCoord OcdFileExport::calculateAreaOffset()
 			// This avoids repeated moves on open/save/close cycles.
 			if (!objects_extent.intersects(ocd_bounds))
 			{
-				addWarning(tr("Coordinates are adjusted to fit into the OCAD 8 drawing area (-2 m ... 2 m)."));
+				addWarning(::OpenOrienteering::OcdFileExport::tr("Coordinates are adjusted to fit into the OCAD 8 drawing area (-2 m ... 2 m)."));
 				std::size_t count = 0;
 				auto calculate_average_center = [&area_offset, &count](const Object* object) {
 					area_offset *= qreal(count)/qreal(count+1);
@@ -680,8 +679,8 @@ MapCoord OcdFileExport::calculateAreaOffset()
 				map->applyOnAllObjects(calculate_average_center);
 			}
 			
-			addWarning(tr("Some coordinates remain outside of the OCAD 8 drawing area."
-			              " They might be unreachable in OCAD."));
+			addWarning(::OpenOrienteering::OcdFileExport::tr("Some coordinates remain outside of the OCAD 8 drawing area."
+			                                                 " They might be unreachable in OCAD."));
 		}
 		
 		if (area_offset.manhattanLength() > 0)
@@ -714,7 +713,7 @@ void OcdFileExport::exportSetup(OcdFile<Ocd::FormatV8>& file)
 		setup->real_offset_y = fields.y;
 		setup->real_angle = fields.a;
 		if (fields.i)
-			addWarning(tr("The georeferencing cannot be saved in OCD version 8."));
+			addWarning(::OpenOrienteering::OcdFileExport::tr("The georeferencing cannot be saved in OCD version 8."));
 		
 		if (view)
 		{
@@ -764,13 +763,13 @@ void OcdFileExport::exportSetup(OcdFile<Ocd::FormatV8>& file)
 		
 		symbol_header.num_separations = spotColorNumber(nullptr);
 		if (symbol_header.num_separations > 24)
-			throw FileFormatException(tr("The map contains more than 24 spot colors which is not supported by OCD version 8."));
+			throw FileFormatException(::OpenOrienteering::OcdFileExport::tr("The map contains more than 24 spot colors which is not supported by OCD version 8."));
 		
 		auto begin_of_spot_colors = beginOfSpotColors(map);
 		if (uses_registration_color)
 			++begin_of_spot_colors;  // in ocd output (ocd_number below)
 		if (begin_of_spot_colors > 256)
-			throw FileFormatException(tr("The map contains more than 256 colors which is not supported by OCD version 8."));
+			throw FileFormatException(::OpenOrienteering::OcdFileExport::tr("The map contains more than 256 colors which is not supported by OCD version 8."));
 		
 		using std::begin; using std::end;
 		auto separation_info = symbol_header.separation_info;
@@ -814,7 +813,7 @@ void OcdFileExport::exportSetup(OcdFile<Ocd::FormatV8>& file)
 				std::fill(begin(color_info->separations), end(color_info->separations), 255);
 				auto index = spotColorNumber(color);
 				if (index >= symbol_header.num_separations)
-					throw FileFormatException(tr("Invalid spot color."));
+					throw FileFormatException(::OpenOrienteering::OcdFileExport::tr("Invalid spot color."));
 				color_info->separations[index] = 200;
 			}
 			else
@@ -826,7 +825,7 @@ void OcdFileExport::exportSetup(OcdFile<Ocd::FormatV8>& file)
 					{
 						auto index = spotColorNumber(component.spot_color);
 						if (index >= symbol_header.num_separations)
-							throw FileFormatException(tr("Invalid spot color."));
+							throw FileFormatException(::OpenOrienteering::OcdFileExport::tr("Invalid spot color."));
 						color_info->separations[index] = quint8(qRound(component.factor * 200));
 					}
 				}
@@ -1040,34 +1039,86 @@ void OcdFileExport::setupBaseSymbol(const Symbol* symbol, quint32 symbol_number,
 	if (symbol->isHidden())
 		ocd_base_symbol.status |= Ocd::SymbolHidden;
 
-	// Set of used colors
+	setupSymbolColors(symbol, ocd_base_symbol);
+	setupIcon(symbol, ocd_base_symbol);
+}
+
+
+template< >
+void OcdFileExport::setupSymbolColors<Ocd::BaseSymbolV8>(const Symbol* symbol, Ocd::BaseSymbolV8& ocd_base_symbol)
+{
 	using ColorBitmask = typename std::remove_extent<typename std::remove_pointer<decltype(ocd_base_symbol.colors)>::type>::type;
 	Q_STATIC_ASSERT(std::is_unsigned<ColorBitmask>::value);
 	ColorBitmask bitmask = 1;
 	
 	auto bitpos = std::begin(ocd_base_symbol.colors);
 	auto last = std::end(ocd_base_symbol.colors);
-	if (uses_registration_color && symbol->containsColor(map->getRegistrationColor()))
+	if (uses_registration_color)
 	{
-		*bitpos |= bitmask;
-		bitmask *= 2;
+		if (symbol->containsColor(map->getRegistrationColor()))
+			*bitpos |= bitmask;
+		bitmask <<= 1;
 	}
 	for (int c = 0; c < map->getNumColors(); ++c)
 	{
 		if (symbol->containsColor(map->getColor(c)))
 			*bitpos |= bitmask;
 		
-		bitmask *= 2;
-		if (bitmask == 0) 
+		if (bitmask == 0x80u) 
 		{
 			bitmask = 1;
-			++bitpos;
 			if (++bitpos == last)
 				break;
 		}
+		else
+		{
+			bitmask <<= 1;
+		}
+	}
+}
+
+template< class OcdBaseSymbol >
+void OcdFileExport::setupSymbolColors(const Symbol* symbol, OcdBaseSymbol& ocd_base_symbol)
+{
+	using std::begin; using std::end;
+	setupSymbolColors(symbol, ocd_base_symbol.num_colors, begin(ocd_base_symbol.colors), end(ocd_base_symbol.colors));
+}
+
+template< typename Counter, typename Iterator >
+void OcdFileExport::setupSymbolColors(const Symbol* symbol, Counter& num_colors, Iterator first, Iterator last)
+{
+	using IndexType = typename std::remove_reference<decltype(*first)>::type;
+	Q_STATIC_ASSERT(std::is_signed<Counter>::value);
+	
+	num_colors = 0;
+	auto it = first;
+	auto registration_offset = 0;
+	
+	if (uses_registration_color)
+	{
+		registration_offset = 1;
+		if (symbol->containsColor(map->getRegistrationColor()))
+		{
+			++num_colors;
+			*it = IndexType(0);
+			++it;
+		}
 	}
 	
-	setupIcon(symbol, ocd_base_symbol);
+	for (int c = 0; c < map->getNumColors(); ++c)
+	{
+		if (!symbol->containsColor(map->getColor(c)))
+			continue;
+		
+		++num_colors;
+		*it = IndexType(c + registration_offset);
+		
+		if (++it == last)
+		{
+			num_colors = Counter(-1);
+			break;
+		}
+	}
 }
 
 
@@ -1297,7 +1348,8 @@ quint8 OcdFileExport::exportAreaSymbolCommon(const AreaSymbol* area_symbol, OcdA
 				}
 				Q_FALLTHROUGH();
 			default:
-				addWarning(tr("In area symbol \"%1\", skipping a fill pattern.").arg(area_symbol->getPlainTextName()));
+				addWarning(::OpenOrienteering::OcdFileExport::tr("In area symbol \"%1\", skipping a fill pattern.")
+				           .arg(area_symbol->getPlainTextName()));
 			}
 			break;
 			
@@ -1322,7 +1374,7 @@ quint8 OcdFileExport::exportAreaSymbolCommon(const AreaSymbol* area_symbol, OcdA
 				// is applicable because with these checks. Already a tiny
 				// (not noticeable) error in the symbol definition would make
 				// it take the wrong choice.
-				addWarning(tr("In area symbol \"%1\", assuming a \"shifted rows\" point pattern. This might be correct as well as incorrect.")
+				addWarning(::OpenOrienteering::OcdFileExport::tr("In area symbol \"%1\", assuming a \"shifted rows\" point pattern. This might be correct as well as incorrect.")
 				           .arg(area_symbol->getPlainTextName()));
 				
 				if (pattern.line_offset != 0)
@@ -1332,7 +1384,8 @@ quint8 OcdFileExport::exportAreaSymbolCommon(const AreaSymbol* area_symbol, OcdA
 				
 				break;
 			default:
-				addWarning(tr("In area symbol \"%1\", skipping a fill pattern.").arg(area_symbol->getPlainTextName()));
+				addWarning(::OpenOrienteering::OcdFileExport::tr("In area symbol \"%1\", skipping a fill pattern.")
+				           .arg(area_symbol->getPlainTextName()));
 			}
 		}
 		
@@ -1454,7 +1507,8 @@ quint32 OcdFileExport::exportLineSymbolCommon(const LineSymbol* line_symbol, Ocd
 		ocd_line_common.line_style = 6;
 	else
 	{
-		addWarning(tr("In line symbol \"%1\", cannot represent cap/join combination.").arg(line_symbol->getPlainTextName()));
+		addWarning(::OpenOrienteering::OcdFileExport::tr("In line symbol \"%1\", cannot represent cap/join combination.")
+		           .arg(line_symbol->getPlainTextName()));
 		// Decide based on the caps
 		if (line_symbol->getCapStyle() == LineSymbol::FlatCap)
 			ocd_line_common.line_style = 0;
@@ -1475,7 +1529,8 @@ quint32 OcdFileExport::exportLineSymbolCommon(const LineSymbol* line_symbol, Ocd
 		if (line_symbol->getMidSymbol() && !line_symbol->getMidSymbol()->isEmpty())
 		{
 			if (line_symbol->getDashesInGroup() > 1)
-				addWarning(tr("In line symbol \"%1\", neglecting the dash grouping.").arg(line_symbol->getPlainTextName()));
+				addWarning(::OpenOrienteering::OcdFileExport::tr("In line symbol \"%1\", neglecting the dash grouping.")
+				           .arg(line_symbol->getPlainTextName()));
 			
 			ocd_line_common.main_length = convertSize(line_symbol->getDashLength() + line_symbol->getBreakLength());
 			ocd_line_common.end_length = ocd_line_common.main_length / 2;
@@ -1486,7 +1541,8 @@ quint32 OcdFileExport::exportLineSymbolCommon(const LineSymbol* line_symbol, Ocd
 			if (line_symbol->getDashesInGroup() > 1)
 			{
 				if (line_symbol->getDashesInGroup() > 2)
-					addWarning(tr("In line symbol \"%1\", the number of dashes in a group has been reduced to 2.").arg(line_symbol->getPlainTextName()));
+					addWarning(::OpenOrienteering::OcdFileExport::tr("In line symbol \"%1\", the number of dashes in a group has been reduced to 2.")
+					           .arg(line_symbol->getPlainTextName()));
 				
 				ocd_line_common.main_length = convertSize(2 * line_symbol->getDashLength() + line_symbol->getInGroupBreakLength());
 				ocd_line_common.end_length = convertSize(2 * line_symbol->getDashLength() + line_symbol->getInGroupBreakLength());
@@ -1580,7 +1636,7 @@ void OcdFileExport::exportLineSymbolDoubleLine(const LineSymbol* line_symbol, qu
 		    || left_border.shift != right_border.shift
 		    || left_border.width != right_border.width)
 		{
-			addWarning(tr("In line symbol \"%1\", cannot export the borders correctly.")
+			addWarning(::OpenOrienteering::OcdFileExport::tr("In line symbol \"%1\", cannot export the borders correctly.")
 			           .arg(line_symbol->getPlainTextName()));
 			
 			if (filling_pattern_differs)
@@ -1771,8 +1827,8 @@ void OcdFileExport::setupTextSymbolBasic(const TextSymbol* text_symbol, int alig
 	ocd_text_basic.font_italic = text_symbol->isItalic() ? 1 : 0;
 	ocd_text_basic.char_spacing = decltype(ocd_text_basic.char_spacing)(convertSize(qRound(1000 * text_symbol->getCharacterSpacing())));
 	if (ocd_text_basic.char_spacing != 0)
-		addWarning(tr("In text symbol %1: custom character spacing is set,"
-		              "its implementation does not match OCAD's behavior yet")
+		addWarning(::OpenOrienteering::OcdFileExport::tr("In text symbol %1: custom character spacing is set,"
+		                                                 "its implementation does not match OCAD's behavior yet")
 		           .arg(text_symbol->getPlainTextName()));
 	ocd_text_basic.word_spacing = 100;
 	ocd_text_basic.alignment = alignment;	// Default value, we might have to change this or even create copies of this symbol with other alignments later
@@ -1787,9 +1843,11 @@ void OcdFileExport::setupTextSymbolSpecial(const TextSymbol* text_symbol, OcdTex
 	ocd_text_special.line_spacing = decltype(ocd_text_special.line_spacing)(qRound(absolute_line_spacing / (text_symbol->getFontSize() * 0.01)));
 	ocd_text_special.para_spacing = convertSize(qRound(1000 * text_symbol->getParagraphSpacing()));
 	if (text_symbol->isUnderlined())
-		addWarning(tr("In text symbol %1: ignoring underlining").arg(text_symbol->getPlainTextName()));
+		addWarning(::OpenOrienteering::OcdFileExport::tr("In text symbol %1: ignoring underlining")
+		           .arg(text_symbol->getPlainTextName()));
 	if (text_symbol->usesKerning())
-		addWarning(tr("In text symbol %1: ignoring kerning").arg(text_symbol->getPlainTextName()));
+		addWarning(::OpenOrienteering::OcdFileExport::tr("In text symbol %1: ignoring kerning")
+		           .arg(text_symbol->getPlainTextName()));
 	
 	ocd_text_special.line_below_on = text_symbol->hasLineBelow() ? 1 : 0;
 	ocd_text_special.line_below_color = convertColor(text_symbol->getLineBelowColor());
@@ -2010,7 +2068,7 @@ void OcdFileExport::exportGenericCombinedSymbol(OcdFile<Format>& file, const Com
 		}
 		if (type == 0)
 		{
-			addWarning(tr("In combined symbol %1: Unsupported subsymbol at index %2.")
+			addWarning(::OpenOrienteering::OcdFileExport::tr("In combined symbol %1: Unsupported subsymbol at index %2.")
 			           .arg(combined_symbol->getPlainTextName(), QString::number(i)));
 		}
 		else if (!combined_symbol->isPartPrivate(i))
@@ -2057,6 +2115,7 @@ QByteArray OcdFileExport::exportCombinedAreaSymbol(
 {
 	auto ocd_symbol = exportAreaSymbol<OcdAreaSymbol>(area_symbol, symbol_number);
 	auto ocd_subsymbol_data = reinterpret_cast<OcdAreaSymbol*>(ocd_symbol.data());
+	setupSymbolColors(combined_symbol, ocd_subsymbol_data->base);
 	setupIcon(combined_symbol, ocd_subsymbol_data->base);
 	ocd_subsymbol_data->common.border_on_V9 = 1;
 	ocd_subsymbol_data->border_symbol = symbol_numbers[line_symbol];
@@ -2074,6 +2133,7 @@ QByteArray OcdFileExport::exportCombinedLineSymbol(
 {
 	auto ocd_symbol = exportLineSymbol<OcdLineSymbol>(main_line, symbol_number);
 	auto ocd_symbol_data = reinterpret_cast<OcdLineSymbol*>(ocd_symbol.data());
+	setupSymbolColors(combined_symbol, ocd_symbol_data->base);
 	setupIcon(combined_symbol, ocd_symbol_data->base);
 	
 	auto& ocd_line_common = ocd_symbol_data->common;
@@ -2090,7 +2150,8 @@ QByteArray OcdFileExport::exportCombinedLineSymbol(
 			ocd_line_common.framing_style = 4;
 		else
 		{
-			addWarning(tr("In line symbol \"%1\", cannot represent cap/join combination.").arg(main_line->getPlainTextName()));
+			addWarning(::OpenOrienteering::OcdFileExport::tr("In line symbol \"%1\", cannot represent cap/join combination.")
+			           .arg(main_line->getPlainTextName()));
 			// Decide based on the caps
 			if (framing->getCapStyle() == LineSymbol::RoundCap)
 				ocd_line_common.framing_style = 1;
@@ -2203,11 +2264,18 @@ void OcdFileExport::exportPathObject(OcdFile<Format>& file, const PathObject* pa
 	
 	bool need_split_lines = false;
 	auto symbol = path->getSymbol();
-	if (symbol &&  symbol->getContainedTypes() & Symbol::Area)
+	if (symbol && symbol->getContainedTypes() & Symbol::Area)
 	{
 		ocd_object.type = 3;  // Area symbol
 		// We we may get away with an object with holes,
 		// but need to check the breakdown.
+		if (symbol->getType() == Symbol::Area)
+		{
+			if (static_cast<const AreaSymbol*>(symbol)->hasRotatableFillPattern())
+				ocd_object.angle = decltype(ocd_object.angle)(convertRotation(path->getPatternRotation()));
+			if (path->getPatternOrigin() != MapCoord(0, 0))
+				addWarning(::OpenOrienteering::OcdFileExport::tr("Unable to export fill pattern shift for an area object"));
+		}
 	}
 	else
 	{
@@ -2405,7 +2473,7 @@ void OcdFileExport::exportTemplates()
 				
 				if (temp->getTemplateState() != Template::Loaded)
 				{
-					addWarning(tr("Unable to save correct position of missing template: \"%1\"")
+					addWarning(::OpenOrienteering::OcdFileExport::tr("Unable to save correct position of missing template: \"%1\"")
 					           .arg(temp->getTemplateFilename()));
 				}
 			}
@@ -2421,7 +2489,8 @@ void OcdFileExport::exportTemplates()
 		}
 		else
 		{
-			addWarning(tr("Unable to export template: file type of \"%1\" is not supported yet").arg(temp->getTemplateFilename()));
+			addWarning(::OpenOrienteering::OcdFileExport::tr("Unable to export template: file type of \"%1\" is not supported yet")
+			           .arg(temp->getTemplateFilename()));
 		}
 	}
 }
@@ -2670,7 +2739,7 @@ QByteArray OcdFileExport::exportTextData(const TextObject* object, int chunk_siz
 void OcdFileExport::addTextTruncationWarning(QString text, int pos)
 {
 	text.insert(pos, QLatin1Char('|'));
-	addWarning(tr("Text truncated at '|'): %1").arg(text));
+	addWarning(::OpenOrienteering::OcdFileExport::tr("Text truncated at '|'): %1").arg(text));
 }
 
 
