@@ -1,5 +1,5 @@
 /*
- *    Copyright 2016-2018 Kai Pastor
+ *    Copyright 2016-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -47,6 +47,8 @@ class OgrTemplate : public TemplateMap
 {
 Q_OBJECT
 public:
+	static bool canRead(const QString& path);
+	
 	static const std::vector<QByteArray>& supportedExtensions();
 	
 	static std::unique_ptr<Georeferencing> getDataGeoreferencing(const QString& path, const Georeferencing& initial_georef);
@@ -54,8 +56,14 @@ public:
 	
 	OgrTemplate(const QString& path, Map* map);
 	
+protected:
+	OgrTemplate(const OgrTemplate& proto);
+	
+public:
 	~OgrTemplate() override;
 	
+	OgrTemplate* duplicate() const override;
+	
 	
 	const char* getTemplateType() const override;
 	
@@ -96,8 +104,6 @@ protected:
 	
 	void mapTransformationChanged();
 	
-	OgrTemplate* duplicateImpl() const override;
-	
 	bool loadTypeSpecificTemplateConfiguration(QXmlStreamReader& xml) override;
 	
 	void saveTypeSpecificTemplateConfiguration(QXmlStreamWriter& xml) const override;
