@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2013-2019 Kai Pastor
+ *    Copyright 2013-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -31,10 +31,12 @@
 #include <Qt>
 #include <QtMath>
 #include <QByteArray>
+#include <QChar>
 #include <QColor>
 #include <QCoreApplication>
 #include <QDir>
 #include <QFileInfo>
+#include <QLatin1Char>
 #include <QLatin1String>
 #include <QMessageBox>
 #include <QPainter>
@@ -50,10 +52,12 @@
 #include "core/map.h"
 #include "core/objects/object.h"
 #include "fileformats/file_format.h"
+#include "gdal/gdal_template.h"
 #include "gdal/ogr_template.h"
 #include "gui/file_dialog.h"
 #include "templates/template_image.h"
 #include "templates/template_map.h"
+#include "templates/template_placeholder.h"
 #include "templates/template_track.h"
 #include "util/backports.h"  // IWYU pragma: keep
 #include "util/util.h"
@@ -219,60 +223,46 @@ bool Template::suppressAbsolutePaths = false;
 
 
 Template::Template(const QString& path, not_null<Map*> map)
- : map(map)
- , template_group(0)
+: map(map)
 {
-	template_path = path;
-	if (! QFileInfo(path).canonicalFilePath().isEmpty())
-		template_path = QFileInfo(path).canonicalFilePath();
-	template_file = QFileInfo(path).fileName();
-	template_relative_path = QString{};
-	template_state = Unloaded;
+	QFileInfo file_info{path};
+	template_file = file_info.fileName();
 	
-	has_unsaved_changes = false;
-	
-	is_georeferenced = false;
-	
-	adjusted = false;
-	adjustment_dirty = true;
+	auto canonical_path = file_info.canonicalFilePath();
+	template_path = canonical_path.isEmpty() ? path : canonical_path;
 	
 	updateTransformationMatrices();
 }
 
+Template::Template(const Template& proto)
+: QObject(nullptr)
+, map(proto.map)
+, template_file(proto.template_file)
+, template_path(proto.template_path)
+, template_relative_path(proto.template_relative_path)
+, template_state(proto.template_state)
+, error_string(proto.error_string)
+, has_unsaved_changes(proto.has_unsaved_changes)
+, is_georeferenced(proto.is_georeferenced)
+, accounted_offset(proto.accounted_offset)
+, transform(proto.transform)
+, other_transform(proto.other_transform)
+, adjusted(proto.adjusted)
+, adjustment_dirty(proto.adjustment_dirty)
+, passpoints(proto.passpoints)
+, template_group(proto.template_group)
+, map_to_template(proto.map_to_template)
+, template_to_map(proto.template_to_map)
+, template_to_map_other(proto.template_to_map_other)
+{
+	// nothing else
+}
+
 Template::~Template()
 {
 	Q_ASSERT(template_state != Loaded);
 }
 
-Template* Template::duplicate() const
-{
-	Template* copy = duplicateImpl();
-	
-	copy->map = map;
-	
-	copy->template_path = template_path;
-	copy->template_relative_path = template_relative_path;
-	copy->template_file = template_file;
-	copy->template_state = template_state;
-	copy->is_georeferenced = is_georeferenced;
-	
-	// Prevent saving the changes twice (if has_unsaved_changes == true)
-	copy->has_unsaved_changes = false;
-	
-	copy->transform = transform;
-	copy->other_transform = other_transform;
-	copy->adjusted = adjusted;
-	copy->adjustment_dirty = adjustment_dirty;
-	
-	copy->map_to_template = map_to_template;
-	copy->template_to_map = template_to_map;
-	copy->template_to_map_other = template_to_map_other;
-	
-	copy->template_group = template_group;
-	
-	return copy;
-}
-
 QString Template::errorString() const
 {
 	return error_string;
@@ -288,6 +278,7 @@ void Template::setErrorString(const QString &text)
 void Template::saveTemplateConfiguration(QXmlStreamWriter& xml, bool open, const QDir* map_dir) const
 {
 	xml.writeStartElement(QString::fromLatin1("template"));
+	xml.writeAttribute(QString::fromLatin1("type"), QString::fromUtf8(getTemplateType()));
 	xml.writeAttribute(QString::fromLatin1("open"), QString::fromLatin1(open ? "true" : "false"));
 	xml.writeAttribute(QString::fromLatin1("name"), getTemplateFilename());
 	auto primary_path = getTemplatePath();
@@ -357,10 +348,13 @@ std::unique_ptr<Template> Template::loadTemplateConfiguration(QXmlStreamReader&
 	if (attributes.hasAttribute(QLatin1String("open")))
 		open = (attributes.value(QLatin1String("open")) == QLatin1String("true"));
 	
+	auto const type = attributes.value(QLatin1String("type"));
 	QString path = attributes.value(QLatin1String("path")).toString();
-	auto temp = templateForFile(path, &map);
+	auto temp = templateForType(type, path, &map);
+	if (!temp && type.length() == 0)
+		temp = templateForPath(path, &map);
 	if (!temp)
-		temp = std::make_unique<TemplateImage>(path, &map); // fallback
+		temp = std::make_unique<TemplatePlaceholder>(type.toUtf8(), path, &map);
 	
 	temp->setTemplateRelativePath(attributes.value(QLatin1String("relpath")).toString());
 	if (attributes.hasAttribute(QLatin1String("name")))
@@ -687,9 +681,9 @@ bool Template::canChangeTemplateGeoreferenced()
 }
 
 // virtual
-bool Template::trySetTemplateGeoreferenced(bool /*value*/, QWidget* /*dialog_parent*/)
+bool Template::trySetTemplateGeoreferenced(bool value, QWidget* /*dialog_parent*/)
 {
-	return is_georeferenced;
+	return isTemplateGeoreferenced() == value;
 }
 
 
@@ -896,51 +890,107 @@ const std::vector<QByteArray>& Template::supportedExtensions()
 		auto& image_extensions = TemplateImage::supportedExtensions();
 		auto& map_extensions   = TemplateMap::supportedExtensions();
 #ifdef MAPPER_USE_GDAL
+		auto& gdal_extensions  = GdalTemplate::supportedExtensions();
 		auto& ogr_extensions   = OgrTemplate::supportedExtensions();
 #else
-		auto ogr_extensions    = std::vector<QByteArray>{ };
+		auto const gdal_extensions = std::vector<QByteArray>{ };
+		auto const& ogr_extensions = gdal_extensions;
 #endif
 		auto& track_extensions = TemplateTrack::supportedExtensions();
 		extensions.reserve(image_extensions.size()
 		                   + map_extensions.size()
+		                   + gdal_extensions.size()
 		                   + ogr_extensions.size()
 		                   + track_extensions.size());
 		extensions.insert(end(extensions), begin(image_extensions), end(image_extensions));
 		extensions.insert(end(extensions), begin(map_extensions), end(map_extensions));
+		extensions.insert(end(extensions), begin(gdal_extensions), end(gdal_extensions));
 		extensions.insert(end(extensions), begin(ogr_extensions), end(ogr_extensions));
 		extensions.insert(end(extensions), begin(track_extensions), end(track_extensions));
 	}
 	return extensions;
 }
 
-std::unique_ptr<Template> Template::templateForFile(const QString& path, Map* map)
+namespace {
+
+bool endsWithAnyOf(const QString& path, const std::vector<QByteArray>& list)
 {
-	auto path_ends_with_any_of = [path](const std::vector<QByteArray>& list) -> bool {
-		using namespace std;
-		return any_of(begin(list), end(list), [path](const QByteArray& extension) {
-			return path.endsWith(QLatin1String(extension), Qt::CaseInsensitive);
-		} );
-	};
+	using namespace std;
+	return any_of(begin(list), end(list), [path](const QByteArray& extension) {
+		return path.size() > extension.size()
+		       && path[path.size() - extension.size() - 1] == QLatin1Char('.')
+		       && path.endsWith(QLatin1String(extension), Qt::CaseInsensitive);
+	} );
+}
+
+}  // namespace
+
+std::unique_ptr<Template> Template::templateForPath(const QString& path, Map* map)
+{
+#ifdef MAPPER_USE_GDAL
+#  define HANDLED_BY_GDAL(expr) expr
+#else
+#  define HANDLED_BY_GDAL(expr) false
+#endif
 	
 	std::unique_ptr<Template> t;
-	if (path_ends_with_any_of(TemplateImage::supportedExtensions()))
+	if (endsWithAnyOf(path, TemplateImage::supportedExtensions())
+	    && !HANDLED_BY_GDAL(endsWithAnyOf(path, GdalTemplate::supportedExtensions())))
 		t = std::make_unique<TemplateImage>(path, map);
-	else if (path_ends_with_any_of(TemplateMap::supportedExtensions()))
+	else if (endsWithAnyOf(path, TemplateMap::supportedExtensions()))
 		t = std::make_unique<TemplateMap>(path, map);
-#ifdef MAPPER_USE_GDAL
-	else if (path_ends_with_any_of(OgrTemplate::supportedExtensions()))
-		t = std::make_unique<OgrTemplate>(path, map);
-#endif
-	else if (path_ends_with_any_of(TemplateTrack::supportedExtensions()))
+	else if (endsWithAnyOf(path, TemplateTrack::supportedExtensions())
+	         && !HANDLED_BY_GDAL(endsWithAnyOf(path, OgrTemplate::supportedExtensions())))
 		t = std::make_unique<TemplateTrack>(path, map);
 #ifdef MAPPER_USE_GDAL
-	else
+	else if (GdalTemplate::canRead(path))
+		t = std::make_unique<GdalTemplate>(path, map);
+	else if (OgrTemplate::canRead(path))
+		t = std::make_unique<OgrTemplate>(path, map);
+	else if (endsWithAnyOf(path, GdalTemplate::supportedExtensions())
+	         && !endsWithAnyOf(path, OgrTemplate::supportedExtensions()))
+		t = std::make_unique<GdalTemplate>(path, map);
+	else if (endsWithAnyOf(path, OgrTemplate::supportedExtensions())
+	         && !endsWithAnyOf(path, GdalTemplate::supportedExtensions()))
 		t = std::make_unique<OgrTemplate>(path, map);
 #endif
 	
 	return t;
 }
 
+std::unique_ptr<Template> Template::templateForType(const QStringRef& type, const QString& path, Map* map)
+{
+	std::unique_ptr<Template> t;
+	
+	// We must respect the customizable handling of tracks,
+	// which is reflected in OgrTemplate::supportedExtensions().
+#ifdef MAPPER_USE_GDAL
+	auto const is_track = endsWithAnyOf(path, TemplateTrack::supportedExtensions());
+	auto const track_with_gdal = (is_track && endsWithAnyOf(path, OgrTemplate::supportedExtensions()));
+#else
+	constexpr auto const track_with_gdal = false;
+#endif
+	
+	auto type_cstring = type.toUtf8();
+	if (type_cstring == "TemplateImage")
+		t = std::make_unique<TemplateImage>(path, map);
+	else if (type_cstring == "TemplateMap")
+		t = std::make_unique<TemplateMap>(path, map);
+#ifdef MAPPER_USE_GDAL
+	else if (type_cstring == "OgrTemplate" && is_track && !track_with_gdal)
+		t = std::make_unique<TemplateTrack>(path, map);
+#endif
+	else if (type_cstring == "TemplateTrack" && !track_with_gdal)
+		t = std::make_unique<TemplateTrack>(path, map);
+#ifdef MAPPER_USE_GDAL
+	else if (type_cstring == "GdalTemplate")
+		t = std::make_unique<GdalTemplate>(path, map);
+	else if (type_cstring == "OgrTemplate" || type_cstring == "TemplateTrack")
+		t = std::make_unique<OgrTemplate>(path, map);
+#endif
+	return t;
+}
+
 
 
 void Template::saveTypeSpecificTemplateConfiguration(QXmlStreamWriter& /*xml*/) const
