@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2019 Kai Pastor
+ *    Copyright 2012-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -30,6 +30,7 @@
 #include <QObject>
 #include <QPointF>
 #include <QString>
+#include <QStringRef>
 
 #include "core/map_coord.h"
 #include "util/matrix.h"
@@ -147,16 +148,19 @@ protected:
 	 * Initializes the template as "Unloaded".
 	 */
 	Template(const QString& path, not_null<Map*> map);
-
+	
+	/**
+	 * Copy-construction as duplication helper.
+	 */
+	Template(const Template& proto);
+	
 public:	
 	~Template() override;
 	
 	/**
 	 * Creates a duplicate of the template
-	 * 
-	 * \todo Rewrite as virtual function, using protected copy constructor.
 	 */
-	Template* duplicate() const;
+	virtual Template* duplicate() const = 0;
 	
 	/**
 	 * Returns a string which should identify the type of the template uniquely:
@@ -258,7 +262,7 @@ public:
 	 *  3. The map directory, if valid, for the filename of the template.
 	 *     This is a fallback for use cases where a map and selected templates
 	 *     are moved to the same flat folder, e.g. when receiving them via
-	 *     individual e-mail attachements.
+	 *     individual e-mail attachments.
 	 * 
 	 * \param map_path  Either the full filepath of the map, or an arbitrary
 	 *                  directory which shall be regarded as the map directory.
@@ -515,12 +519,11 @@ public:
 	 * Tries to change the usage of georeferencing data.
 	 * 
 	 * If supported by the actual template, this function tries to switch the
-	 * state between non-georeferenced and georeferenced. It returns the final
-	 * state which may be the same as before if the change is not implemented
-	 * or fails for other reasons.
+	 * state between non-georeferenced and georeferenced. It returns false when
+	 * an error occurred, and true if the change was successful or if it was
+	 * explicitly cancelled by the user.
 	 * 
-	 * The default implementation changes nothing, and it just returns the
-	 * current state.
+	 * The default implementation returns true iff the state matches the value.
 	 */
 	virtual bool trySetTemplateGeoreferenced(bool value, QWidget* dialog_parent);
 	
@@ -574,10 +577,20 @@ public:
 	 * Creates a Template instance for the given path.
 	 * 
 	 * This function tries to find a matching template subclass by looking at
-	 * the file extension. It may return nullptr if no subclass supports the
-	 * extension.
+	 * the file extension and/or the content available at the given path.
+	 * It may return nullptr if no subclass supports the extension or content.
 	 */
-	static std::unique_ptr<Template> templateForFile(const QString& path, Map* map);
+	static std::unique_ptr<Template> templateForPath(const QString& path, Map* map);
+	
+	/**
+	 * Creates a Template instance for the given type.
+	 * 
+	 * This function may return nullptr if the given type is unknown.
+	 * 
+	 * In addition, this function respects the user setting for assigning
+	 * some TemplateTrack extensions (GPX) explicitly to GDAL (OgrTemplate).
+	 */
+	static std::unique_ptr<Template> templateForType(const QStringRef& type, const QString& path, Map* map);
 	
 	/**
 	 * A flag which disables the writing of absolute paths for template files.
@@ -605,18 +618,7 @@ protected:
 	void setErrorString(const QString &text);
 	
 	
-	/** 
-	 * Derived classes must create a duplicate and transfer
-	 * 
-	 * type specific information over to the copy here.
-	 * This includes the content of the template file if it is loaded.
-	 * 
-	 * \todo Rewrite together with duplicate().
-	 */
-	virtual Template* duplicateImpl() const = 0;
-	
-	
-	/** 
+	/**
 	 * Hook for saving parameters needed by the actual template type.
 	 * 
 	 * The default implementation does nothing.
@@ -681,16 +683,16 @@ protected:
 	QString template_relative_path;
 	
 	/// The template lifetime state
-	State template_state;
+	State template_state = Unloaded;
 	
 	/// The description of the last error
 	QString error_string;
 	
 	/// Does the template itself (not its transformation) have unsaved changes (e.g. GPS track has changed, image has been painted on)
-	bool has_unsaved_changes;
+	bool has_unsaved_changes = false;
 	
 	/// Is the template in georeferenced mode?
-	bool is_georeferenced;
+	bool is_georeferenced = false;
 	
 private:	
 	// Properties for non-georeferenced templates (invalid if is_georeferenced is true) 
@@ -714,16 +716,17 @@ protected:
 	TemplateTransform other_transform;
 	
 	/// If true, transform is the adjusted transformation, otherwise it is the original one
-	bool adjusted;
+	bool adjusted = false;
 	
 	/// If true, the adjusted transformation has to be recalculated
-	bool adjustment_dirty;
+	bool adjustment_dirty = true;
 	
 	/// List of pass points for position adjustment
 	PassPointList passpoints;
 	
 	/// Number of the template group. If the template is not grouped, this is set to -1.
-	int template_group;
+	/// \todo Switch to initialization with -1. ATM 0 is kept for compatibility.
+	int template_group = 0;
 	
 	// Transformation matrices calculated from cur_trans
 	Matrix map_to_template;
