@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2019 Kai Pastor
+ *    Copyright 2012-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -23,6 +23,7 @@
 
 #include <iosfwd>
 #include <iterator>
+#include <utility>
 
 #include <Qt>
 #include <QtGlobal>
@@ -31,9 +32,9 @@
 #include <QDialog>
 #include <QFileInfo>  // IWYU pragma: keep
 #include <QImageReader>
+#include <QImageWriter>
 #include <QLatin1String>
 #include <QList>
-#include <QMessageBox>
 #include <QPaintEngine>
 #include <QPainter>
 #include <QPen>
@@ -59,13 +60,18 @@
 #include "util/transformation.h"
 #include "util/util.h"
 
-#ifdef MAPPER_USE_GDAL
-#include "gdal/gdal_template.h"
-#endif
-
 
 namespace OpenOrienteering {
 
+#ifdef MAPPER_USE_GDAL
+
+// Forward declaration, from "gdal/gdal_image_reader.h",
+// to avoid a direct dependency on GDAL API includes.
+TemplateImage::GeoreferencingOption readGdalGeoTransform(const QString& filepath);
+
+#endif
+
+
 const std::vector<QByteArray>& TemplateImage::supportedExtensions()
 {
 	static std::vector<QByteArray> extensions;
@@ -79,22 +85,41 @@ const std::vector<QByteArray>& TemplateImage::supportedExtensions()
 	return extensions;
 }
 
-TemplateImage::TemplateImage(const QString& path, Map* map) : Template(path, map)
+TemplateImage::TemplateImage(const QString& path, Map* map)
+: Template(path, map)
+, georef(std::make_unique<Georeferencing>())
 {
-	undo_index = 0;
-	georef.reset(new Georeferencing());
-	available_georef.push_back({Georeferencing_None, "no georeferencing information", {}, {}});
-	
 	const Georeferencing& georef = map->getGeoreferencing();
 	connect(&georef, &Georeferencing::projectionChanged, this, &TemplateImage::updateGeoreferencing);
 	connect(&georef, &Georeferencing::transformationChanged, this, &TemplateImage::updateGeoreferencing);
 }
+
+TemplateImage::TemplateImage(const TemplateImage& proto)
+: Template(proto)
+, image(proto.image)
+// not copied: undo_steps
+// not copied: undo_index
+, available_georef(proto.available_georef)
+, georef(new Georeferencing(*proto.georef))
+{
+	const Georeferencing& georef = map->getGeoreferencing();
+	connect(&georef, &Georeferencing::projectionChanged, this, &TemplateImage::updateGeoreferencing);
+	connect(&georef, &Georeferencing::transformationChanged, this, &TemplateImage::updateGeoreferencing);
+}
+
 TemplateImage::~TemplateImage()
 {
 	if (template_state == Loaded)
 		unloadTemplateFile();
 }
 
+
+TemplateImage* TemplateImage::duplicate() const
+{
+	return new TemplateImage(*this);
+}
+
+
 bool TemplateImage::saveTemplateFile() const
 {
 	const auto result = image.save(template_path);
@@ -113,8 +138,8 @@ void TemplateImage::saveTypeSpecificTemplateConfiguration(QXmlStreamWriter& xml)
 	{
 		// Follow map georeferencing XML structure
 		xml.writeStartElement(QString::fromLatin1("crs_spec"));
-// TODO: xml.writeAttribute(QString::fromLatin1("language"), "PROJ.4");
-		xml.writeCharacters(temp_crs_spec);
+		/// \todo xml.writeAttribute(QString::fromLatin1("language"), "PROJ.4");
+		xml.writeCharacters(available_georef.effective.crs_spec);
 		xml.writeEndElement(/*crs_spec*/);
 	}
 }
@@ -123,8 +148,8 @@ bool TemplateImage::loadTypeSpecificTemplateConfiguration(QXmlStreamReader& xml)
 {
 	if (is_georeferenced && xml.name() == QLatin1String("crs_spec"))
 	{
-		// TODO: check specification language
-		temp_crs_spec = xml.readElementText();
+		/// \todo check specification language
+		available_georef.effective.crs_spec = xml.readElementText();
 	}
 	else
 		xml.skipCurrentElement(); // unsupported
@@ -135,6 +160,10 @@ bool TemplateImage::loadTypeSpecificTemplateConfiguration(QXmlStreamReader& xml)
 bool TemplateImage::loadTemplateFileImpl(bool configuring)
 {
 	QImageReader reader(template_path);
+	
+	// QImageReader::format() cannot be called after reading.
+	drawable = QImageWriter::supportedImageFormats().contains(reader.format());
+	
 	const QSize size = reader.size();
 	const QImage::Format format = reader.imageFormat();
 	if (size.isEmpty() || format == QImage::Format_Invalid)
@@ -161,11 +190,15 @@ bool TemplateImage::loadTemplateFileImpl(bool configuring)
 		return false;
 	}
 	
-	available_georef = findAvailableGeoreferencing();
+#ifdef MAPPER_USE_GDAL
+	available_georef = findAvailableGeoreferencing(readGdalGeoTransform(template_path));
+#else
+	available_georef = findAvailableGeoreferencing({});
+#endif
 	
 	if (!configuring && is_georeferenced)
 	{
-		if (available_georef.front().type == Georeferencing_None)
+		if (!isGeoreferencingUsable())
 		{
 			// Image was georeferenced, but georeferencing info is gone -> deny to load template
 			setErrorString(tr("Georeferencing not found"));
@@ -177,11 +210,9 @@ bool TemplateImage::loadTemplateFileImpl(bool configuring)
 	
 	return true;
 }
-bool TemplateImage::postLoadConfiguration(QWidget* dialog_parent, bool& /*out_center_in_view*/)
+
+bool TemplateImage::postLoadConfiguration(QWidget* dialog_parent, bool& out_center_in_view)
 {
-	if (getTemplateFilename().endsWith(QLatin1String(".gif"), Qt::CaseInsensitive))
-		QMessageBox::warning(dialog_parent, tr("Warning"), tr("Loading a GIF image template.\nSaving GIF files is not supported. This means that drawings on this template won't be saved!\nIf you do not intend to draw on this template however, that is no problem."));
-	
 	TemplateImageOpenDialog open_dialog(this, dialog_parent);
 	open_dialog.setWindowModality(Qt::WindowModal);
 	while (true)
@@ -192,15 +223,11 @@ bool TemplateImage::postLoadConfiguration(QWidget* dialog_parent, bool& /*out_ce
 		if (!open_dialog.isGeorefRadioChecked())
 			break;
 		
-		Q_ASSERT(!available_georef.empty());  // implied by open_dialog.isGeorefRadioChecked()
-		temp_crs_spec = available_georef.front().crs_spec;
 		if (map->getGeoreferencing().isLocal())
 		{
 			// Make sure that the map is georeferenced.
 			Georeferencing initial_georef(map->getGeoreferencing());
-			if (!initial_georef.setProjectedCRS({}, temp_crs_spec) || initial_georef.isLocal())
-				temp_crs_spec.clear();
-			
+			initial_georef.setProjectedCRS({}, available_georef.effective.crs_spec);
 			if (initial_georef.getMapRefPoint() == MapCoord{}
 			    && initial_georef.getProjectedRefPoint() == QPointF{})
 			{
@@ -208,6 +235,8 @@ bool TemplateImage::postLoadConfiguration(QWidget* dialog_parent, bool& /*out_ce
 				calculateGeoreferencing();
 				auto const center_pixel = MapCoordF(0.5 * (image.width() - 1), 0.5 * (image.height() - 1));
 				initial_georef.setProjectedRefPoint(georef->toProjectedCoords(center_pixel));
+				initial_georef.setCombinedScaleFactor(1.0);
+				initial_georef.setGrivation(0.0);
 			}
 			
 			GeoreferencingDialog dialog(dialog_parent, map, &initial_georef, false);
@@ -217,36 +246,49 @@ bool TemplateImage::postLoadConfiguration(QWidget* dialog_parent, bool& /*out_ce
 				dialog.setKeepGeographicRefCoords();
 			if (dialog.exec() == QDialog::Rejected)
 				continue;
+			
+			if (map->getGeoreferencing().getProjectedCRSSpec() == available_georef.effective.crs_spec)
+			{
+				// For convenience, skip CRS selection.
+				break;
+			}
 		}
 		
-		if (temp_crs_spec.isEmpty())
+		if (!map->getGeoreferencing().isLocal())
 		{
-			// Let the user select the coordinate reference system.
+			// Let user select the coordinate reference system.
+			// \todo Change description text below (no longer just for world files.)
+			Q_UNUSED(QT_TR_NOOP("Select the coordinate reference system of the georeferenced image."))
 			SelectCRSDialog dialog(
+			            available_georef,
 			            map->getGeoreferencing(),
 			            dialog_parent,
-			            SelectCRSDialog::TakeFromMap | SelectCRSDialog::Geographic,
 			            tr("Select the coordinate reference system of the coordinates in the world file") );
-			if (dialog.exec() == QDialog::Rejected)
-				continue;
-			temp_crs_spec = dialog.currentCRSSpec();
+			if (dialog.exec() == QDialog::Accepted)
+			{
+				available_georef.effective.crs_spec = dialog.currentCRSSpec();
+				break;
+			}
 		}
-		
-		break;
 	}
 	
-	is_georeferenced = open_dialog.isGeorefRadioChecked();	
-	if (is_georeferenced)
+	if (open_dialog.isGeorefRadioChecked())
+	{
 		calculateGeoreferencing();
+		// If not both the template and the map are fully georeferenced,
+		// we use the transform, but don't claim the georeferenced state.
+		is_georeferenced = georef->isValid()
+		                   && !georef->isLocal()
+		                   && map->getGeoreferencing().isValid()
+		                   && !map->getGeoreferencing().isLocal();
+		out_center_in_view = false;
+	}
 	else
 	{
+		is_georeferenced = false;
 		transform.template_scale_x = open_dialog.getMpp() * 1000.0 / map->getScaleDenominator();
 		transform.template_scale_y = transform.template_scale_x;
 		transform.template_shear = 0.0;
-		
-		//transform.template_x = main_view->getPositionX();
-		//transform.template_y = main_view->getPositionY();
-		
 		updateTransformationMatrices();
 	}
 	
@@ -321,7 +363,8 @@ QPointF TemplateImage::calcCenterOfGravity(QRgb background_color)
 
 bool TemplateImage::canChangeTemplateGeoreferenced()
 {
-	return available_georef.front().type != Georeferencing_None;
+	// No need to care for CRS here and now: This is handled by the dialog ATM.
+	return !available_georef.effective.transform.source.isEmpty();
 }
 
 bool TemplateImage::trySetTemplateGeoreferenced(bool value, QWidget* dialog_parent)
@@ -335,32 +378,35 @@ bool TemplateImage::trySetTemplateGeoreferenced(bool value, QWidget* dialog_pare
 		if (value)
 		{
 			// Cf. postLoadConfiguration
-			if (available_georef.front().type == Georeferencing_WorldFile
-			    && temp_crs_spec.isEmpty())
-			{
-				// Let user select the coordinate reference system, as this is not specified in world files
-				SelectCRSDialog dialog(
-				            map->getGeoreferencing(),
-				            dialog_parent,
-				            SelectCRSDialog::TakeFromMap | SelectCRSDialog::Geographic,
-				            tr("Select the coordinate reference system of the coordinates in the world file") );
-				if (dialog.exec() == QDialog::Rejected)
-					return is_georeferenced;
-				temp_crs_spec = dialog.currentCRSSpec();
-			}
+			// Let user select the coordinate reference system.
+			// \todo Change description text below (no longer just for world files.)
+			Q_UNUSED(QT_TR_NOOP("Select the coordinate reference system of the georeferenced image."))
+			SelectCRSDialog dialog(
+			            available_georef,
+			            map->getGeoreferencing(),
+			            dialog_parent,
+			            tr("Select the coordinate reference system of the coordinates in the world file") );
+			if (dialog.exec() == QDialog::Rejected)
+				return true;  // not failed!
 			
 			setTemplateAreaDirty();
-			is_georeferenced = true;
+			available_georef.effective.crs_spec = dialog.currentCRSSpec();
 			calculateGeoreferencing();
+			// If not both the template and the map are fully georeferenced,
+			// we use the transform, but don't claim the georeferenced state.
+			is_georeferenced = georef->isValid()
+			                   && !georef->isLocal()
+			                   && map->getGeoreferencing().isValid()
+			                   && !map->getGeoreferencing().isLocal();
 			setTemplateAreaDirty();
 		}
 		else
 		{
 			is_georeferenced = false;
 		}
-		setHasUnsavedChanges(true);
+		map->setTemplatesDirty();
 	}
-	return is_georeferenced;
+	return isTemplateGeoreferenced() == value;
 }
 
 void TemplateImage::updateGeoreferencing()
@@ -369,61 +415,86 @@ void TemplateImage::updateGeoreferencing()
 		updatePosFromGeoreferencing();
 }
 
-TemplateImage::GeoreferencingOptions TemplateImage::findAvailableGeoreferencing() const
+
+namespace {
+
+TemplateImage::GeoreferencingOption worldFileOption(const QString& template_path)
 {
-	GeoreferencingOptions result;
-	
-	auto crs_spec = temp_crs_spec; // loaded or empty
-#ifdef MAPPER_USE_GDAL
-	auto const gdal_georef = GdalTemplate::tryReadProjection(template_path);
-	auto const proj_spec = QString::fromUtf8(GdalTemplate::RasterGeoreferencing::toProjSpec(gdal_georef.spec));
-	if (crs_spec.isEmpty())
-		crs_spec = proj_spec; // loaded or empty
-#endif
-	
+	auto option = TemplateImage::GeoreferencingOption {};
 	WorldFile world_file;
 	if (world_file.tryToLoadForImage(template_path))
 	{
-		auto pixel_to_world = QTransform(world_file);
-		if (!crs_spec.isEmpty())
-		{
-			Georeferencing tmp_georef;
-			tmp_georef.setProjectedCRS(QString{}, crs_spec);
-			if (tmp_georef.isGeographic())
-			{
-				constexpr auto factor = qDegreesToRadians(1.0);
-				pixel_to_world = {
-				    pixel_to_world.m11() * factor, pixel_to_world.m12() * factor, 0,
-				    pixel_to_world.m21() * factor, pixel_to_world.m22() * factor, 0,
-				    pixel_to_world.m31() * factor, pixel_to_world.m32() * factor, 1
-				};
-			}
-		}
-		result.push_back({Georeferencing_WorldFile, "World file", crs_spec, pixel_to_world});
+		option.transform.source = "World file";
+		option.transform.pixel_to_world = QTransform(world_file);
 	}
-	
-#ifdef MAPPER_USE_GDAL
-	if (gdal_georef.valid)
-	{
-		result.push_back({Georeferencing_GDAL, gdal_georef.driver, proj_spec, QTransform(gdal_georef)});
-	}
-#endif
-	
-	Q_ASSERT(available_georef.back().type == Georeferencing_None);
-	result.push_back(available_georef.back());
-	
-	return result;
+	return option;
 }
 
+}  // namespace
 
-Template* TemplateImage::duplicateImpl() const
+
+TemplateImage::GeoreferencingOptions TemplateImage::findAvailableGeoreferencing(GeoreferencingOption template_file_option) const
 {
-	auto new_template = new TemplateImage(template_path, map);
-	new_template->image = image;
-	new_template->available_georef = available_georef;
-	return new_template;
+	auto options = GeoreferencingOptions {
+		{},
+		worldFileOption(template_path),
+		std::move(template_file_option),
+	};
+	
+	// Try to select the effective CRS from:
+	// (1) current effective CRS, (2) template file, (3) map
+	if (!available_georef.effective.crs_spec.isEmpty())
+	{
+		options.effective.crs_spec = available_georef.effective.crs_spec;
+	}
+	else if (!options.template_file.crs_spec.isEmpty())
+	{
+		options.effective.crs_spec = options.template_file.crs_spec;
+	}
+	else if (!map->getGeoreferencing().getProjectedCRSSpec().isEmpty())
+	{
+		options.effective.crs_spec = map->getGeoreferencing().getProjectedCRSSpec();
+	}
+	
+	// Try to select the effective transform from:
+	// (1) world file, (2) template file
+	if (!options.world_file.transform.source.isEmpty())
+	{
+		options.effective.transform = options.world_file.transform;
+	}
+	else if (!options.template_file.transform.source.isEmpty())
+	{
+		options.effective.transform = options.template_file.transform;
+	}
+	
+	// Modify transform from geographic CRS
+	if (!options.effective.crs_spec.isEmpty() && !options.effective.transform.source.isEmpty())
+	{
+		Georeferencing tmp_georef;
+		tmp_georef.setProjectedCRS(QString{}, options.effective.crs_spec);
+		if (tmp_georef.isGeographic())
+		{
+			constexpr auto factor = qDegreesToRadians(1.0);
+			auto& pixel_to_world = options.effective.transform.pixel_to_world;
+			pixel_to_world = {
+			    pixel_to_world.m11() * factor, pixel_to_world.m12() * factor, 0,
+			    pixel_to_world.m21() * factor, pixel_to_world.m22() * factor, 0,
+			    pixel_to_world.m31() * factor, pixel_to_world.m32() * factor, 1
+			};
+		}
+	}
+	
+	return options;
 }
 
+bool TemplateImage::isGeoreferencingUsable() const
+{
+	return !available_georef.effective.transform.source.isEmpty()
+	       && (!available_georef.effective.crs_spec.isEmpty()
+	           || map->getGeoreferencing().getProjectedCRSSpec().isEmpty());
+}
+
+
 void TemplateImage::drawOntoTemplateImpl(MapCoordF* coords, int num_coords, const QColor& color, qreal width)
 {
 	QPointF* points;
@@ -565,36 +636,18 @@ void TemplateImage::addUndoStep(const TemplateImage::DrawOnImageUndoStep& new_st
 
 void TemplateImage::calculateGeoreferencing()
 {
-	// Calculate georeferencing of image coordinates where the coordinate (0, 0)
-	// is mapped to the world position of the top-left corner of the top-left pixel
-	applyGeoreferencingOption(available_georef.front());
-}
-
-void TemplateImage::applyGeoreferencingOption(const GeoreferencingOption& option)
-{
-	if (option.type == Georeferencing_None)
+	if (!isGeoreferencingUsable())
 	{
-		qWarning("%s shall not be called for Georeferencing_None", Q_FUNC_INFO);
+		qWarning("%s must not be called with incomplete georeferencing", Q_FUNC_INFO);
 		georef->setTransformationDirectly({});
 		return;
 	}
 	
-	georef.reset(new Georeferencing());
-	switch (option.type)
-	{
-	case Georeferencing_WorldFile:
-		if (!temp_crs_spec.isEmpty())
-			georef->setProjectedCRS(QString::fromUtf8(option.source), temp_crs_spec);
-		break;
-	case Georeferencing_GDAL:
-#ifdef MAPPER_USE_GDAL
-		georef->setProjectedCRS(QString::fromUtf8(option.source), option.crs_spec);
-#endif		
-		break;
-	case Georeferencing_None:
-		break;
-	}
-	georef->setTransformationDirectly(option.pixel_to_world);
+	// Calculate georeferencing of image coordinates where the coordinate (0, 0)
+	// is mapped to the world position of the top-left corner of the top-left pixel
+	georef = std::make_unique<Georeferencing>();
+	georef->setProjectedCRS(QString{}, available_georef.effective.crs_spec);
+	georef->setTransformationDirectly(available_georef.effective.transform.pixel_to_world);
 	if (map->getGeoreferencing().isValid())
 		updatePosFromGeoreferencing();
 }
@@ -604,19 +657,19 @@ void TemplateImage::updatePosFromGeoreferencing()
 	// Determine map coords of three image corner points
 	// by transforming the points from one Georeferencing into the other
 	bool ok;
-	MapCoordF top_left = map->getGeoreferencing().toMapCoordF(georef.data(), MapCoordF(0.0, 0.0), &ok);
+	MapCoordF top_left = map->getGeoreferencing().toMapCoordF(georef.get(), MapCoordF(0.0, 0.0), &ok);
 	if (!ok)
 	{
 		qDebug("%s failed", Q_FUNC_INFO);
 		return; // TODO: proper error message?
 	}
-	MapCoordF top_right = map->getGeoreferencing().toMapCoordF(georef.data(), MapCoordF(image.width(), 0.0), &ok);
+	MapCoordF top_right = map->getGeoreferencing().toMapCoordF(georef.get(), MapCoordF(image.width(), 0.0), &ok);
 	if (!ok)
 	{
 		qDebug("%s failed", Q_FUNC_INFO);
 		return; // TODO: proper error message?
 	}
-	MapCoordF bottom_left = map->getGeoreferencing().toMapCoordF(georef.data(), MapCoordF(0.0, image.height()), &ok);
+	MapCoordF bottom_left = map->getGeoreferencing().toMapCoordF(georef.get(), MapCoordF(0.0, image.height()), &ok);
 	if (!ok)
 	{
 		qDebug("%s failed", Q_FUNC_INFO);
