@@ -56,9 +56,11 @@ const std::vector<QByteArray>& TemplateMap::supportedExtensions()
 
 TemplateMap::TemplateMap(const QString& path, Map* map)
 : Template(path, map)
-{
-	// nothing
-}
+{}
+
+TemplateMap::TemplateMap(const TemplateMap& proto)
+: Template(proto)
+{}
 
 TemplateMap::~TemplateMap()
 {
@@ -66,6 +68,15 @@ TemplateMap::~TemplateMap()
 		unloadTemplateFile();
 }
 
+TemplateMap* TemplateMap::duplicate() const
+{
+	auto* copy = new TemplateMap(*this);
+	if (template_state == Loaded)
+		copy->loadTemplateFileImpl(false);
+	return copy;
+}
+
+
 const char* TemplateMap::getTemplateType() const
 {
 	return "TemplateMap";
@@ -180,14 +191,6 @@ QRectF TemplateMap::getTemplateExtent() const
 	return extent;
 }
 
-Template* TemplateMap::duplicateImpl() const
-{
-	auto copy = new TemplateMap(template_path, map);
-	if (template_state == Loaded)
-		copy->loadTemplateFileImpl(false);
-	return copy;
-}
-
 
 bool TemplateMap::hasAlpha() const
 {
